# -*- coding: utf-8 -*-
import unittest2 as unittest
from plone.resource.testing import PLONE_RESOURCE_INTEGRATION_TESTING
from plone.resource.interfaces import IResourceDirectory


class IntegrationTestCase(unittest.TestCase):
    layer = PLONE_RESOURCE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer.get('portal')

    def test_persistent_directory_installed(self):
        # directory should be available as the portal_resources tool
        from Products.CMFCore.utils import getToolByName
        tool = getToolByName(self.portal, 'portal_resources')
        self.assertEqual('portal_resources', tool.getId())

        # wrapper should be available as an IResourceDirectory utility named 'persistent'
        from zope.component import getUtility
        utility = getUtility(IResourceDirectory, name='persistent')

        # the utility's context attribute is the (unwrapped) tool
        from Acquisition import aq_base
        self.assertTrue(aq_base(tool) is utility.context)



from plone.resource.interfaces import IPloneResourceCreatedEvent
from plone.resource.interfaces import IPloneResourceModifiedEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implementer


@implementer(IPloneResourceCreatedEvent)
class PloneResourceCreatedEvent(ObjectEvent):
    """A resource has been created"""


@implementer(IPloneResourceModifiedEvent)
class PloneResourceModifiedEvent(ObjectEvent):
    """A resource has been modified"""



# -*- coding: utf-8 -*-
from StringIO import StringIO
from zope.publisher.browser import BrowserView


class DownloadView(BrowserView):

    def __call__(self):
        name = self.context.__name__

        response = self.request.response

        # XXX: It would be better to have the zip file written straight to
        # the output stream, but this is awkward with the ZPublisher response
        # interface. For now, we write the zipfile to a stream in memory.

        out = StringIO()
        self.context.exportZip(out)

        response.setHeader('Content-Type', 'application/zip')
        response.setHeader(
            'Content-Disposition',
            'attachment; filename="%s.zip"' % name
        )
        response.setHeader('Content-Length', len(out.getvalue()))

        response.write(out.getvalue())



# -*- coding: utf-8 -*-
import unittest2 as unittest
from zope.component.testing import tearDown
from zope.configuration.exceptions import ConfigurationError

import os.path
from StringIO import StringIO
from zope.configuration.xmlconfig import xmlconfig, XMLConfig
from zope.component import getUtility
from plone.resource.interfaces import IResourceDirectory


def clearZCML(test=None):
    # Copy from ``zope.component.tests.examples``
    from zope.configuration.xmlconfig import XMLConfig
    import zope.component
    from zope.component.testing import setUp
    from zope.component.testing import tearDown
    tearDown()
    setUp()
    XMLConfig('meta.zcml', zope.component)()


def runSnippet(snippet, dist='plone.resource.tests'):
    template = """\
    <configure xmlns="http://namespaces.zope.org/zope"
               xmlns:plone="http://namespaces.plone.org/plone"
               i18n_domain="plone"
               %s>
    %s
    </configure>"""
    dist = 'package="%s"' % dist if dist else ''
    xmlconfig(StringIO(template % (dist, snippet)))


class ZCMLTestCase(unittest.TestCase):

    def setUp(self):
        clearZCML()
        import plone.resource
        XMLConfig('meta.zcml', plone.resource)()

    def tearDown(self):
        tearDown()

    def test_dist_with_name_and_type(self):
        runSnippet("""
        <plone:static
          name="foo"
          type="theme"
          directory="resources"
          />
        """)

        res = getUtility(IResourceDirectory, name='++theme++foo')
        self.assertTrue(res.directory.endswith(os.path.join('plone', 'resource', 'tests', 'resources')))

    def test_dist_rejects_with_missing_type(self):
        # resource directories in distributions must be registered with a type
        self.assertRaises(ConfigurationError,
            runSnippet,
            """<plone:static
              name="foo"
              directory="resources"
              />"""
            )

    def test_dist_with_type_only(self):
        runSnippet("""
        <plone:static
          type="theme"
          directory="resources"
          />
        """)

        getUtility(IResourceDirectory, name='++theme++plone.resource.tests')

    def test_dist_rejects_absolute_directory(self):
        self.assertRaises(ConfigurationError,
            runSnippet,
            """<plone:static directory="/" />"""
            )

    def test_global(self):
        runSnippet("""
        <plone:static
          directory="/"
          />
        """, dist=None)

        res = getUtility(IResourceDirectory)
        self.assertEqual('/', res.directory)

    def test_global_rejects_relative_directory(self):
        self.assertRaises(ConfigurationError,
            runSnippet,
            """<plone:static directory="foobar" />""",
            dist=None
            )

    def test_missing_directory(self):
        self.assertRaises(ConfigurationError,
            runSnippet,
            """<plone:static directory="foobar" />"""
            )

    def test_rejects_parent_directory_traversal(self):
        self.assertRaises(ConfigurationError,
            runSnippet,
            """<plone:static directory="../tests" />"""
            )



# -*- coding: utf-8 -*-
from zope.interface import Attribute
from zope.interface import Interface
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.publisher.interfaces import IPublishTraverse


class IResourceDirectory(IPublishTraverse):
    """A directory of file-like resources.

    This interface provides a common API for interacting with resources
    regardless of whether they are stored on the filesystem or in the
    ZODB.
    """

    __name__ = Attribute("""The name of the directory.""")

    def __repr__():
        """Returns a string identifier of the directory."""

    def __contains__(name):
        """Return true if the given file or directory exists
        """

    def __getitem__(name):
        """Return the file or resource directory with the given name
        as an object
        """

    def openFile(path):
        """Returns the file or filelike object identified by the given path
        (relative to this directory).

        Raises IOError if the file cannot be opened.
        """

    def readFile(path):
        """Returns the contents of the file identified by the given path.

        Raises IOError if the file cannot be read.
        """

    def listDirectory():
        """Lists the contents of this directory.

        Raises OSError if the directory cannot be read.
        """

    def isDirectory(path):
        """Returns True if the given path (relative to this directory) is a
        directory (as opposed to a file).
        """

    def isFile(path):
        """Returns True if the given path is a file."""

    def exportZip(out):
        """Exports the contents of this directory as a zip file, which will
        be written to the open file handle ``out``.
        """


class IWritableResourceDirectory(IResourceDirectory):

    def makeDirectory(path):
        """Create the given path as a directory. (Returns successfully without
        doing anything if the directory already exists.)
        """

    def writeFile(path, data):
        """Write a file at the specified path.

        Parent directories will be added if necessary. The final path component
        gives the filename. If the file already exists, it will be overwritten.

        ``data`` may be a string or file-like object.
        """

    def importZip(file):
        """Imports the contents of a zip file into this directory.

        ``file`` may be a filename, file-like object, or instance of
        zipfile.ZipFile. The file data must be a ZIP archive.
        """

    def __delitem__(name):
        """Delete a file or directory inside this directory
        """

    def __setitem__(name, item):
        """Add a file or directory as returned by ``__getitem__()``
        """

    def rename(oldName, newName):
        """Rename a child file or folder
        """


class IUniqueResourceRequest(Interface):
    """Marker interface for requests to ++unique++<id>"""


class IPloneResourceCreatedEvent(IObjectCreatedEvent):
    """An resource has been created."""


class IPloneResourceModifiedEvent(IObjectModifiedEvent):
    """An resource has been created."""



# -*- coding: utf-8 -*-
from plone.resource.interfaces import IResourceDirectory
from zExceptions import NotFound
from zope.component import getUtilitiesFor
from zope.component import queryUtility


def iterDirectoriesOfType(type, filter_duplicates=True):
    """
    Returns a generator which iterates over all resource directories of a
    particular resource type.

    If ``filter_duplicates`` is True and multiple resource directory trees
    contain resource directories with identical names, only the
    first one found will be returned for each name. The following sources are
    checked in order:
    - the persistent portal_resources tool
    - the global resource directory
    - resource directories in distributions
    """

    found = set()

    # 1. Persistent resource directory:
    #    List (persistent resource directory)/$type
    res = queryUtility(IResourceDirectory, name=u'persistent')
    if res and res.isDirectory(type):
        typedir = res[type]
        for dirname in typedir.listDirectory():
            if not typedir.isDirectory(dirname):
                continue
            yield typedir[dirname]
            found.add(dirname)

    # 2. Global resource directory:
    #    List (global resource directory)/$type
    res = queryUtility(IResourceDirectory, name=u'')
    if res and res.isDirectory(type):
        typedir = res[type]
        for dirname in typedir.listDirectory():
            if not typedir.isDirectory(dirname):
                continue
            if not filter_duplicates or dirname not in found:
                yield typedir[dirname]
                found.add(dirname)

    # 3. Packaged resource directories:
    #    Scan the registry
    identifier = '++%s++' % type
    for name, u in getUtilitiesFor(IResourceDirectory):
        if name.startswith(identifier):
            if not filter_duplicates or u.__name__ not in found:
                yield u


def queryResourceDirectory(type, name):
    """Find the IResourceDirectory of the given name and type. Returns
    None if not found.
    """

    # 1. Persistent resource directory:
    #    Try (persistent resource directory)/$type/$name
    res = queryUtility(IResourceDirectory, name=u'persistent')
    if res:
        try:
            return res[type][name]
        except (KeyError, NotFound,):
            pass  # pragma: no cover

    # 2. Global resource directory:
    #    Try (global resource directory)/$type/$name
    res = queryUtility(IResourceDirectory, name=u'')
    if res:
        try:
            return res[type][name]
        except (KeyError, NotFound,):
            pass  # pragma: no cover

    # 3. Packaged type-specific resource directory:
    #    Try (directory named after type + name)
    identifier = u'++%s++%s' % (type, name)
    res = queryUtility(IResourceDirectory, name=identifier)
    if res is not None:
        return res

    return None


def cloneResourceDirectory(source, target):
    """Copy all directories and files in the resource directory source to
    the writable resource directory target
    """

    for name in source.listDirectory():
        if source.isDirectory(name):
            target.makeDirectory(name)
            cloneResourceDirectory(source[name], target[name])
        else:
            f = source.openFile(name)
            try:
                target.writeFile(name, f)
            finally:
                f.close()




# -*- coding: utf-8 -*-
import unittest2 as unittest
from plone.testing import zca
from plone.resource.testing import DEMO_TRAVERSER_INTEGRATION_TESTING

import os.path
from zope.component import provideUtility
from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2
from plone.resource.interfaces import IResourceDirectory
from plone.resource.directory import PersistentResourceDirectory
from plone.resource.directory import FilesystemResourceDirectory

base_path = os.path.dirname(__file__)
test_dir_path = os.path.join(base_path, 'resources')


class TraversalTestCase(unittest.TestCase):
    layer = DEMO_TRAVERSER_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer.get('app')
        zca.pushGlobalRegistry()

        # set up all three types of directory
        package_dir_path = os.path.join(test_dir_path, 'demo', 'foo')
        self.package_dir = dir = FilesystemResourceDirectory(package_dir_path)
        provideUtility(dir, provides=IResourceDirectory, name=u'++demo++foo')

        self.global_dir = dir = FilesystemResourceDirectory(test_dir_path)
        provideUtility(dir, provides=IResourceDirectory, name=u'')

        root = BTreeFolder2('portal_resources')
        self.app._setOb('portal_resources', root)
        root._setOb('demo', BTreeFolder2('demo'))
        root.demo._setOb('foo', BTreeFolder2('foo'))
        self.zodb_dir = dir = PersistentResourceDirectory(root)
        provideUtility(dir, provides=IResourceDirectory, name=u'persistent')

        # We don't want a false positive for the following.
        provideUtility(dir, provides=IResourceDirectory, name=u'++bogus++foo')

    def tearDown(self):
        zca.popGlobalRegistry()

    def test_iterDirectoriesOfType(self):
        from plone.resource.utils import iterDirectoriesOfType
        dirs = list(iterDirectoriesOfType('demo'))
        self.assertEqual(2, len(dirs))
        self.assertTrue(dirs[0].context.aq_base is
                        self.zodb_dir['demo']['foo'].context.aq_base)
        self.assertTrue(dirs[1].directory ==
                        self.global_dir['demo']['manifest-test'].directory)

    def test_iterDirectoriesOfType_dont_filter_duplicates(self):
        from plone.resource.utils import iterDirectoriesOfType
        dirs = list(iterDirectoriesOfType('demo', filter_duplicates=False))
        self.assertEqual(4, len(dirs))
        self.assertTrue(dirs[0].context.aq_base is
                        self.zodb_dir['demo']['foo'].context.aq_base)
        unordered_entries = [
            self.global_dir['demo']['foo'].directory,
            self.global_dir['demo']['manifest-test'].directory
        ]
        self.assertIn(dirs[1].directory, unordered_entries)
        self.assertIn(dirs[2].directory, unordered_entries)
        self.assertNotEqual(dirs[1].directory, dirs[2].directory)
        self.assertTrue(dirs[3].directory == self.package_dir.directory)

    def test_cloneDirectory(self):
        from plone.resource.directory import PersistentResourceDirectory
        from plone.resource.utils import cloneResourceDirectory

        root = BTreeFolder2('portal_resources')
        root._setOb('demo', BTreeFolder2('demo'))
        root['demo']._setOb('foo', BTreeFolder2('foo'))
        root['demo']._setOb('bar', BTreeFolder2('bar'))

        source = PersistentResourceDirectory(root['demo']['foo'])
        target = PersistentResourceDirectory(root['demo']['bar'])

        source.writeFile('file1.txt', 'file1')
        source.writeFile('subdir1/file2.txt', 'file2')
        source.makeDirectory('subdir2')

        cloneResourceDirectory(source, target)

        self.assertEqual(source.listDirectory(), target.listDirectory())
        self.assertEqual(source['subdir1'].listDirectory(), target['subdir1'].listDirectory())
        self.assertEqual(source['subdir2'].listDirectory(), target['subdir2'].listDirectory())
        self.assertEqual(source.readFile('file1.txt'), target.readFile('file1.txt'))
        self.assertEqual(source.readFile('subdir1/file2.txt'), target.readFile('subdir1/file2.txt'))



# -*- coding: utf-8 -*-
"""Utilities for working with manifest files.

The manifest is stored in a file ``manifest.cfg`` in the root of a resource
directory. It uses ini file (ConfigParser) syntax.

For example, for the resource type foo, there could be a manifest file like::

    [foo]
    title = A foo resource
    description = Used for doing foo-like things
    bar = Moe's

We can define this manifest format using::

    from plone.resource.manifest import ManifestFormat
    FOO_FORMAT = ManifestFormat('foo',
            keys=['title', 'description', 'bar'],
            defaults={'bar': 'baz'},
        )

To get this manifest from an open file pointer ``fp``, do::

    from plone.resource.manifest import getManifest
    manifest = getManifest(fp, FOO_FORMAT)

``manifest`` will now be a dict with keys ``title``, ``description``, and
``bar``. ``title`` and ``description`` will be ``None`` if not found in the
manifest. ``bar`` will be ``baz`` if not found.
"""
from ConfigParser import SafeConfigParser
from plone.resource.directory import FILTERS
from plone.resource.interfaces import IResourceDirectory
from plone.resource.utils import iterDirectoriesOfType
from zope.component import getUtility
import logging

MANIFEST_FILENAME = 'manifest.cfg'

LOGGER = logging.getLogger('plone.resource.manifest')


class ManifestFormat(object):
    """Describes a manifest format.

    An immutable, threadsafe object.

    ``resourceType`` is used as the section header.

    ``keys`` should be a list of keys that should be returned.

    ``defaults`` can be used to pass a dict of default values. The keys
    should correspond to ``keys``, but it is not mandatory to fill every key.

    ``parameterSections`` can be a list section names in the ``manifest.cfg``
    file that can be used to supply additional, free-form parameters. For
    example, if ``parameters`` is ['parameters'] and 'resourceType' is
    'theme', then the ``manifest.cfg`` file may optionally contain a section
    ``[theme:parameters]``.
    """

    def __init__(self, resourceType, keys, defaults=None,
                 parameterSections=None):
        self.resourceType = resourceType
        self.keys = keys
        self.defaults = defaults or {}
        self.parameterSections = parameterSections or []


def getManifest(fp, format, defaults=None):
    """Read the manifest from the given open file pointer according to the
    given ManifestFormat. Pass a dict as ``defaults`` to override the defaults
    from the manifest format.
    """

    if defaults is None:
        defaults = format.defaults

    parser = SafeConfigParser()
    parser.readfp(fp)

    results = {}
    for key in format.keys:
        if parser.has_option(format.resourceType, key):
            results[key] = parser.get(format.resourceType, key)
        else:
            results[key] = defaults.get(key, None)

    for key in format.parameterSections:
        sectionName = "%s:%s" % (format.resourceType, key,)
        if parser.has_section(sectionName):
            results[key] = dict(parser.items(sectionName))
        else:
            results[key] = {}

    return results


def extractManifestFromZipFile(zipfile, format, defaults=None,
                               manifestFilename=MANIFEST_FILENAME):
    """Get a resource name and manifest from the given open
    ``zipfile.ZipFile`` using the given format. Returns a tuple::

        (resourceName, manifestDict)

    Where ``resourceName`` is the resource name, taken to be the name of the
    single top level directory inside the zip file (ignoring certain OS
    files), and ``manifestDict`` is a dictionary as returned by
    ``getManifest()``.

    The ``manifestDict`` may be None if no manifest file was found.

    Will throw a ValueError if the zip file does not contain a single top
    level directory.

    Pass ``defaults`` to override the defaults from the manifest format.

    Pass ``manifestFilename`` to use a custom manifest filename.
    """

    resourceName = None
    manifestDict = None

    for name in zipfile.namelist():
        member = zipfile.getinfo(name)
        path = member.filename.lstrip('/')

        # Skip filtered files (OS X junk and dot files, mainly)
        if any(
            any(
                filter.match(n) for filter in FILTERS
            ) for n in path.split('/')
        ):
            continue

        pathSegments = path.rstrip('/').split('/')
        isDirectory = path.endswith('/')

        # Is this a new top level directory?
        if pathSegments[0] != resourceName:

            # We already thought we had one - abort
            if resourceName is not None:
                raise ValueError("More than one top level directory")

            # Store the first path segment (which may either be the directory)
            # itself, or the directory part in which some file is found
            if isDirectory or len(pathSegments) > 0:
                resourceName = pathSegments[0]
            else:
                raise ValueError(
                    "Found a top level file - expected a single top level "
                    "directory"
                )

        # Did we find a manifest file?
        if (
            resourceName is not None and
            not isDirectory and
            path == "%s/%s" % (resourceName, manifestFilename,)
        ):
            manifest = zipfile.open(member)
            try:
                manifestDict = getManifest(manifest, format, defaults=defaults)
            finally:
                manifest.close()

    if resourceName is None:
        raise ValueError("No top level directory found")

    return (resourceName, manifestDict)


def getAllResources(format, defaults=None, filter=None,
                    manifestFilename=MANIFEST_FILENAME):
    """Get a dict of all resources of the resource type indicated by the
    manifest format. Returns a dict where the keys are the resource ids and
    the values are manifests. The value may be None if no manifest was found.

    Pass ``defaults`` to override the defaults from the manifest format.

    Pass ``filter``, a callable that takes a resource directory as its
    only argument, if you want to be able to filter out any resource
    directories. It should return True if the given directory should be
    included.

    Pass ``manifestFilename`` to use a different manifest file name
    convention.
    """

    resources = {}

    for directory in iterDirectoriesOfType(format.resourceType):

        if filter is not None and not filter(directory):
            continue

        name = directory.__name__
        resources[name] = None

        if directory.isFile(manifestFilename):

            manifest = directory.openFile(manifestFilename)
            try:
                resources[name] = getManifest(manifest, format, defaults)
            except:
                LOGGER.exception(
                    "Unable to read manifest for theme directory {0}".format(
                        name
                    )
                )
            finally:
                manifest.close()

    return resources


def getZODBResources(format, defaults=None, filter=None,
                     manifestFilename=MANIFEST_FILENAME):
    """Get a dict of all resources in the ZODB of the resource type indicated
    by the manifest format. Returns a dict where the keys are the resource
    ids and the values are manifests. The value may be None if no manifest was
    found.

    Pass ``defaults`` to override the defaults from the manifest format.

    Pass ``filter``, a callable that takes a resource directory as its
    only argument, if you want to be able to filter out any resource
    directories. It should return True if the given directory should be
    included.

    Pass ``manifestFilename`` to use a different manifest file name
    convention.
    """

    resources = {}

    persistentDirectory = getUtility(IResourceDirectory, name="persistent")
    if format.resourceType not in persistentDirectory:
        return resources

    resourcesDirectory = persistentDirectory[format.resourceType]

    for name in resourcesDirectory.listDirectory():

        resourceDir = resourcesDirectory[name]

        if filter is not None and not filter(resourceDir):
            continue

        resources[name] = None

        if resourceDir.isFile(manifestFilename):
            manifest = resourceDir.openFile(MANIFEST_FILENAME)
            try:
                resources[name] = getManifest(manifest, format, defaults)
            except:
                LOGGER.exception(
                    "Unable to read manifest for {0} directory {1}".format(
                        manifest.resourceType, name
                    )
                )
            finally:
                manifest.close()

    return resources



# -*- coding: utf-8 -*-
import unittest2 as unittest
from plone.testing import zca

from plone.resource.testing import DEMO_TRAVERSER_INTEGRATION_TESTING

import zipfile
import os.path

from zope.component import provideUtility

from plone.resource.interfaces import IResourceDirectory

from plone.resource.manifest import ManifestFormat
from plone.resource.manifest import getManifest
from plone.resource.manifest import extractManifestFromZipFile
from plone.resource.manifest import getAllResources
from plone.resource.manifest import getZODBResources

from plone.resource.directory import FilesystemResourceDirectory
from plone.resource.directory import PersistentResourceDirectory

from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2

base_path = os.path.dirname(__file__)
test_dir_path = os.path.join(base_path, 'resources')

TEST_FORMAT = ManifestFormat('demo', ['title', 'description', 'bar'], {'bar': 'baz'})
TEST_FORMAT_PARAMS = ManifestFormat('demo', ['title', 'description', 'bar'], {'bar': 'baz'}, ['params'])

class TestManifest(unittest.TestCase):
    layer = DEMO_TRAVERSER_INTEGRATION_TESTING

    def setUp(self):
        zca.pushGlobalRegistry()

    def tearDown(self):
        zca.popGlobalRegistry()

    def test_get_manifest(self):
        fp = open(os.path.join(test_dir_path, 'demo', 'manifest-test', 'manifest.cfg'))
        manifest = getManifest(fp, TEST_FORMAT)
        self.assertEqual(manifest['title'], 'Manifest test')
        self.assertEqual(manifest['description'], None)
        self.assertEqual(manifest['bar'], 'baz')

        fp.close()

    def test_get_manifest_params(self):
        fp = open(os.path.join(test_dir_path, 'demo', 'manifest-test', 'manifest.cfg'))
        manifest = getManifest(fp, TEST_FORMAT_PARAMS)
        self.assertEqual(manifest['title'], 'Manifest test')
        self.assertEqual(manifest['description'], None)
        self.assertEqual(manifest['bar'], 'baz')
        self.assertEqual(manifest['params'], {'alpha': 'beta', 'delta': 'theta'})

        fp.close()

    def test_get_manifest_ignores_extra(self):
        fp = open(os.path.join(test_dir_path, 'demo', 'manifest-test', 'manifest.cfg'))
        manifest = getManifest(fp, TEST_FORMAT)
        self.assertFalse('baz' in manifest)
        fp.close()

    def test_get_manifest_override_defaults(self):
        fp = open(os.path.join(test_dir_path, 'demo', 'manifest-test', 'manifest.cfg'))
        manifest = getManifest(fp, TEST_FORMAT, {'bar': 'foo', 'title': 'foo'})
        self.assertEqual(manifest['title'], 'Manifest test')
        self.assertEqual(manifest['bar'], 'foo')
        fp.close()

    def test_extract_from_zip_file(self):
        zf = zipfile.ZipFile(os.path.join(base_path, 'zipfiles', 'normal.zip'))
        resourceName, manifestDict = extractManifestFromZipFile(zf, TEST_FORMAT)

        self.assertEqual(resourceName, 'demo1')
        self.assertEqual(
                manifestDict,
                {'bar': 'baz', 'description': None, 'title': 'No top level dir'}
            )

    def test_extract_from_zip_file_override_defaults(self):
        zf = zipfile.ZipFile(os.path.join(base_path, 'zipfiles', 'normal.zip'))
        resourceName, manifestDict = extractManifestFromZipFile(zf, TEST_FORMAT,
            defaults={'bar': 'foo', 'description': 'desc'})

        self.assertEqual(resourceName, 'demo1')
        self.assertEqual(
                manifestDict,
                {'bar': 'foo', 'description': 'desc', 'title': 'No top level dir'}
            )

    def test_extract_from_zip_file_no_top_level_dir(self):
        zf = zipfile.ZipFile(os.path.join(base_path, 'zipfiles', 'no-top-level-dir.zip'))
        self.assertRaises(ValueError, extractManifestFromZipFile, zf, TEST_FORMAT)

    def test_extract_from_zip_file_multiple_top_level_dirs(self):
        zf = zipfile.ZipFile(os.path.join(base_path, 'zipfiles', 'multiple-top-level-dirs.zip'))
        self.assertRaises(ValueError, extractManifestFromZipFile, zf, TEST_FORMAT)

    def test_extract_from_zip_file_no_manifest(self):
        zf = zipfile.ZipFile(os.path.join(base_path, 'zipfiles', 'no-manifest.zip'))
        resourceName, manifestDict = extractManifestFromZipFile(zf, TEST_FORMAT)

        self.assertEqual(resourceName, 'demo1')
        self.assertEqual(manifestDict, None)

    def test_extract_from_zip_file_manifest_name_override(self):
        zf = zipfile.ZipFile(os.path.join(base_path, 'zipfiles', 'manifest-name-override.zip'))
        resourceName, manifestDict = extractManifestFromZipFile(zf, TEST_FORMAT)

        self.assertEqual(resourceName, 'demo1')
        self.assertEqual(manifestDict, None)

        resourceName, manifestDict = extractManifestFromZipFile(zf, TEST_FORMAT, manifestFilename='other-manifest.cfg')

        self.assertEqual(
                manifestDict,
                {'bar': 'baz', 'description': None, 'title': 'No top level dir'}
            )

    def test_get_all_resources(self):
        app = self.layer['app']

        foo = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'foo'))
        provideUtility(foo, provides=IResourceDirectory, name=u'++demo++foo')

        manifestTest = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'manifest-test'))
        provideUtility(manifestTest, provides=IResourceDirectory, name=u'++demo++manifest-test')

        root = BTreeFolder2('portal_resources')
        app._setOb('portal_resources', root)
        root._setOb('demo', BTreeFolder2('demo'))
        root['demo']._setOb('bar', BTreeFolder2('bar'))

        persistentDir = PersistentResourceDirectory(root)
        provideUtility(persistentDir, provides=IResourceDirectory, name=u'persistent')

        resources = getAllResources(TEST_FORMAT)

        self.assertEqual(
                resources,
                {'bar': None,
                 'foo': None,
                 'manifest-test': {'bar': 'baz',
                                   'description': None,
                                   'title': 'Manifest test'}}
            )

    def test_get_all_resources_filter(self):
        app = self.layer['app']

        foo = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'foo'))
        provideUtility(foo, provides=IResourceDirectory, name=u'++demo++foo')

        manifestTest = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'manifest-test'))
        provideUtility(manifestTest, provides=IResourceDirectory, name=u'++demo++manifest-test')

        root = BTreeFolder2('portal_resources')
        app._setOb('portal_resources', root)
        root._setOb('demo', BTreeFolder2('demo'))
        root['demo']._setOb('bar', BTreeFolder2('bar'))

        persistentDir = PersistentResourceDirectory(root)
        provideUtility(persistentDir, provides=IResourceDirectory, name=u'persistent')

        resources = getAllResources(TEST_FORMAT, filter=lambda dir: dir.__name__ != 'foo')

        self.assertEqual(
                resources,
                {'bar': None,
                 'manifest-test': {'bar': 'baz',
                                   'description': None,
                                   'title': 'Manifest test'}}
            )

    def test_get_zodb_resources(self):
        app = self.layer['app']

        foo = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'foo'))
        provideUtility(foo, provides=IResourceDirectory, name=u'++demo++foo')

        manifestTest = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'manifest-test'))
        provideUtility(manifestTest, provides=IResourceDirectory, name=u'++demo++manifest-test')

        root = BTreeFolder2('portal_resources')
        app._setOb('portal_resources', root)
        root._setOb('demo', BTreeFolder2('demo'))
        root['demo']._setOb('bar', BTreeFolder2('bar'))
        root['demo']._setOb('baz', BTreeFolder2('baz'))

        persistentDir = PersistentResourceDirectory(root)
        provideUtility(persistentDir, provides=IResourceDirectory, name=u'persistent')

        resources = getZODBResources(TEST_FORMAT)

        self.assertEqual(
                resources,
                {'bar': None,
                 'baz': None}
            )

    def test_get_zodb_resources_filter(self):
        app = self.layer['app']

        foo = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'foo'))
        provideUtility(foo, provides=IResourceDirectory, name=u'++demo++foo')

        manifestTest = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'manifest-test'))
        provideUtility(manifestTest, provides=IResourceDirectory, name=u'++demo++manifest-test')

        root = BTreeFolder2('portal_resources')
        app._setOb('portal_resources', root)
        root._setOb('demo', BTreeFolder2('demo'))
        root['demo']._setOb('bar', BTreeFolder2('bar'))
        root['demo']._setOb('baz', BTreeFolder2('baz'))

        persistentDir = PersistentResourceDirectory(root)
        provideUtility(persistentDir, provides=IResourceDirectory, name=u'persistent')

        resources = getZODBResources(TEST_FORMAT, filter=lambda dir: dir.__name__ != 'baz')

        self.assertEqual(
                resources,
                {'bar': None}
            )



# -*- coding: utf-8 -*-
import unittest2 as unittest
from plone.testing import zca, z2
from plone.resource.testing import DEMO_TRAVERSER_INTEGRATION_TESTING

import os.path
from zExceptions import NotFound
from zope.component import provideUtility
from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2
from plone.resource.interfaces import IResourceDirectory
from plone.resource.interfaces import IUniqueResourceRequest
from plone.resource.directory import PersistentResourceDirectory
from plone.resource.directory import FilesystemResourceDirectory
from plone.resource.file import FilesystemFile
from zExceptions import Forbidden

base_path = os.path.dirname(__file__)
test_dir_path = os.path.join(base_path, 'resources')


class TraversalTestCase(unittest.TestCase):
    layer = DEMO_TRAVERSER_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer.get('app')
        zca.pushGlobalRegistry()

    def tearDown(self):
        zca.popGlobalRegistry()

    def test_traverse_packaged_type_specific_directory(self):
        dir = FilesystemResourceDirectory(test_dir_path)
        provideUtility(dir, provides=IResourceDirectory, name=u'++demo++foo')

        res = self.app.restrictedTraverse('++demo++foo')
        self.assertTrue(res.directory.endswith('resources'))

        self.assertRaises(NotFound, self.app.restrictedTraverse, '++demo++asdf')

    def test_traverse_packaged_type_specific_file(self):
        dir = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'foo'))
        provideUtility(dir, provides=IResourceDirectory, name=u'++demo++foo')

        res = self.app.restrictedTraverse('++demo++foo/test.html')
        self.assertTrue(isinstance(res, FilesystemFile))

    def test_traverse_global_directory(self):
        dir = FilesystemResourceDirectory(test_dir_path)
        provideUtility(dir, provides=IResourceDirectory, name=u'')

        res = self.app.restrictedTraverse('++demo++foo')
        self.assertTrue(res.directory.endswith('resources/demo/foo'))

        self.assertRaises(NotFound, self.app.restrictedTraverse, '++demo++bar')

    def test_traverse_persistent_directory(self):
        root = BTreeFolder2('portal_resources')
        self.app._setOb('portal_resources', root)
        root._setOb('demo', BTreeFolder2('demo'))
        root.demo._setOb('foo', BTreeFolder2('foo'))

        dir = PersistentResourceDirectory(root)
        provideUtility(dir, provides=IResourceDirectory, name=u'persistent')

        res = self.app.restrictedTraverse('++demo++foo')
        self.assertEqual('portal_resources/demo/foo', '/'.join(res.context.getPhysicalPath()))

        self.assertRaises(NotFound, self.app.restrictedTraverse, '++demo++bar')

    def test_publish_resource(self):
        dir = FilesystemResourceDirectory(test_dir_path)
        provideUtility(dir, provides=IResourceDirectory, name=u'')

        browser = z2.Browser(self.app)
        browser.handleErrors = False

        browser.open(self.app.absolute_url() + '/++demo++foo/test.html')
        self.assertEqual('asdf', browser.contents)

    def test_traverse_unique_resource_marks_request(self):
        dir = FilesystemResourceDirectory(test_dir_path)
        provideUtility(dir, provides=IResourceDirectory, name=u'')

        res = self.app.restrictedTraverse('++demo++foo/++unique++bar/test.html')
        self.assertTrue(IUniqueResourceRequest.providedBy(self.app.REQUEST))

    def test_publish_unique_resource(self):
        dir = FilesystemResourceDirectory(test_dir_path)
        provideUtility(dir, provides=IResourceDirectory, name=u'')

        browser = z2.Browser(self.app)
        browser.handleErrors = False

        browser.open(self.app.absolute_url() + '/++demo++foo/++unique++bar/test.html')
        self.assertEqual('asdf', browser.contents)

    def test_forbidden_resource_path_traversal(self):
        resource_directory = FilesystemResourceDirectory(test_dir_path)
        self.assertRaises(Forbidden, resource_directory._resolveSubpath,
                          '../../../../setup.py')



# -*- coding: utf-8 -*-
import os.path
import unittest2 as unittest
from Acquisition import aq_base
from plone.resource.events import PloneResourceCreatedEvent
from plone.resource.events import PloneResourceModifiedEvent
from plone.resource.interfaces import IPloneResourceCreatedEvent
from plone.resource.interfaces import IPloneResourceModifiedEvent
from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2
from zope.component import adapter
from zope.component import provideHandler

class TestPersistentResourceDirectory(unittest.TestCase):

    def _makeOne(self):
        root = BTreeFolder2('portal_resources')
        root._setOb('demo', BTreeFolder2('demo'))
        root.demo._setOb('foo', BTreeFolder2('foo'))

        from StringIO import StringIO
        from OFS.Image import File
        file = File('test.html', 'test.html', StringIO('asdf'))
        root.demo.foo._setOb('test.html', file)

        from plone.resource.directory import PersistentResourceDirectory
        return PersistentResourceDirectory(root)

    def _assertSameUnwrapped(self, a, b):
        self.assertTrue(aq_base(a) is aq_base(b))

    def test_ctor_implicit_context(self):
        from zope.site.testing import siteSetUp, createSiteManager, siteTearDown
        siteSetUp()

        site = BTreeFolder2('site')
        createSiteManager(site, setsite=True)
        root = self._makeOne().context
        site._setOb('portal_resources', root)

        from plone.resource.directory import PersistentResourceDirectory
        try:
            dir = PersistentResourceDirectory()
            # context should be stored unwrapped
            self.assertTrue(aq_base(root) is dir.context)

            # but re-wrapped during traversal
            traversed = dir['demo']
            self.assertTrue(site in traversed.context.aq_chain)
        finally:
            siteTearDown()

    def test_repr(self):
        dir = self._makeOne()
        s = '<PersistentResourceDirectory object at portal_resources>'
        self.assertEqual(s, repr(dir))

    def test_publishTraverse_directory(self):
        dir = self._makeOne()
        subdir = dir.publishTraverse(None, 'demo')
        self._assertSameUnwrapped(subdir.context, dir.context.demo)

    def test_publishTraverse_file(self):
        dir = self._makeOne()
        file = dir.publishTraverse(None, 'demo/foo/test.html')
        self._assertSameUnwrapped(file, dir.context.demo.foo['test.html'])

    def test_publishTraverse_not_found(self):
        dir = self._makeOne()
        from zExceptions import NotFound
        self.assertRaises(NotFound, dir.publishTraverse, None, 'baz')

    def test_getitem(self):
        dir = self._makeOne()
        self._assertSameUnwrapped(dir['demo'].context, dir.context.demo)

    def test_openFile(self):
        dir = self._makeOne()
        file = dir.openFile('demo/foo/test.html')
        self.assertEqual('asdf', file.read())

    def test_readFile(self):
        dir = self._makeOne()
        self.assertEqual('asdf', dir.readFile('demo/foo/test.html'))

    def test_readFile_not_found(self):
        dir = self._makeOne()
        self.assertRaises(IOError, dir.readFile, 'baz')

    def test_listDirectory(self):
        dir = self._makeOne()
        self.assertEqual(['demo'], dir.listDirectory())

    def test_listDirectory_filters_by_name(self):
        dir = self._makeOne()
        dir.context._setOb('.svn', BTreeFolder2('filtered'))
        self.assertEqual(['demo'], dir.listDirectory())

    def test_makeDirectory(self):
        dir = self._makeOne()
        dir.makeDirectory('demo/bar')
        newdir = dir['demo']['bar']
        self.assertTrue(isinstance(newdir.context, BTreeFolder2))

    def test_makeDirectory_extra_slashes(self):
        dir = self._makeOne()
        dir.makeDirectory('/demo/bar/')
        newdir = dir['demo']['bar']
        self.assertTrue(isinstance(newdir.context, BTreeFolder2))

    def test_writeFile(self):
        dir = self._makeOne()
        dir.writeFile('qux', 'qux')
        self.assertEqual('qux', dir.readFile('qux'))

    def test_writeFile_does_not_create_empty_directory(self):
        dir = self._makeOne()
        dir.writeFile('qux', 'qux')
        self.assertFalse('' in dir)
        self.assertTrue('qux' in dir)

    def test_writeFile_directory_missing(self):
        dir = self._makeOne()
        dir.writeFile('baz/qux', 'qux')
        self.assertEqual('qux', dir.readFile('baz/qux'))

    def test_writeFile_file_already_exists(self):
        dir = self._makeOne()
        dir.writeFile('demo/foo/test.html', 'changed')
        self.assertEqual('changed', dir.readFile('demo/foo/test.html'))

    def test_importZip(self):
        dir = self._makeOne()
        f = open(os.path.join(os.path.dirname(__file__), 'resources.zip'))
        dir.importZip(f)
        self.assertEqual('from zip', dir.readFile('demo/foo/test.html'))

    def test_importZip_takes_ZipFile(self):
        dir = self._makeOne()
        from zipfile import ZipFile
        f = ZipFile(os.path.join(os.path.dirname(__file__), 'resources.zip'))
        dir.importZip(f)
        self.assertEqual('from zip', dir.readFile('demo/foo/test.html'))

    def test_importZip_filters_resource_forks(self):
        dir = self._makeOne()
        f = open(os.path.join(os.path.dirname(__file__), 'resources.zip'))
        dir.importZip(f)
        self.assertFalse('__MACOSX' in dir.context.objectIds())

    def test_importZip_filters_hidden_directories(self):
        dir = self._makeOne()
        f = open(os.path.join(os.path.dirname(__file__), 'resources.zip'))
        dir.importZip(f)
        self.assertFalse('.svn' in dir)

    def test_delitem(self):
        dir = self._makeOne()
        dir.makeDirectory('demo')
        self.assertTrue('demo' in dir)
        del dir['demo']
        self.assertFalse('demo' in dir)

    def test_rename(self):
        dir = self._makeOne()
        dir.rename('demo', 'demo1')
        self.assertEqual(['demo1'], dir.listDirectory())

    def test_setitem_file(self):
        dir = self._makeOne()
        f = dir['demo']['foo']['test.html']
        dir['demo'].makeDirectory('bar')

        dir['demo']['bar']['test.html'] = f
        self.assertEqual(dir['demo']['foo'].readFile('test.html'),
                         dir['demo']['bar'].readFile('test.html'),)

    def test_setitem_file_unicode(self):
        dir = self._makeOne()
        f = dir['demo']['foo']['test.html']
        dir['demo'].makeDirectory('bar')

        dir['demo']['bar'][u'test.html'] = f
        self.assertEqual(dir['demo']['foo'].readFile('test.html'),
                         dir['demo']['bar'].readFile('test.html'),)

    def test_setitem_directory(self):
        dir = self._makeOne()
        dir['demo']['foo'].makeDirectory('d1')

        d1 = dir['demo']['foo']['d1']
        del dir['demo']['foo']['d1']

        dir['demo']['foo']['d2'] = d1

        self.assertEqual(dir['demo']['foo']['d2'].__name__, 'd2')

    def test_events(self):
        events = []

        @adapter(IPloneResourceCreatedEvent)
        def _handleFileCreated(event):
            events.append(event)
        provideHandler(_handleFileCreated)

        @adapter(IPloneResourceModifiedEvent)
        def _handleFileModified(event):
            events.append(event)
        provideHandler(_handleFileModified)

        dir = self._makeOne()
        dir.writeFile('test', 'my test')
        dir.writeFile('test', 'my test is modified')
        self.assertTrue(isinstance(events[0], PloneResourceCreatedEvent))
        self.assertEqual(
            str(events[0].object),
            'my test'
        )
        self.assertTrue(isinstance(events[1], PloneResourceModifiedEvent))
        self.assertEqual(
            str(events[1].object),
            'my test is modified'
        )


class TestFilesystemResourceDirectory(unittest.TestCase):

    def _makeOne(self, name=None):
        from plone.resource.directory import FilesystemResourceDirectory
        path = os.path.join(os.path.dirname(__file__), 'resources')
        return FilesystemResourceDirectory(path, name=name)

    def test_repr(self):
        dir = self._makeOne()
        s = '<FilesystemResourceDirectory object at resources>'
        self.assertEqual(s, repr(dir))

    def test_repr(self):
        dir = self._makeOne(name='something-else')
        s = '<FilesystemResourceDirectory object at something-else>'
        # This used to give a ValueError: substring not found
        self.assertEqual(s, repr(dir))

    def test_publishTraverse_directory(self):
        dir = self._makeOne()
        subdir = dir.publishTraverse(None, 'demo')
        self.assertEqual(subdir.directory, os.path.join(dir.directory, 'demo'))

    def test_publishTraverse_file(self):
        dir = self._makeOne()
        file = dir.publishTraverse(None, 'demo/foo/test.html')
        subpath = os.path.join(dir.directory, 'demo', 'foo', 'test.html')
        self.assertEqual(file.path, subpath)

    def test_publishTraverse_not_found(self):
        dir = self._makeOne()
        from zExceptions import NotFound
        self.assertRaises(NotFound, dir.publishTraverse, None, 'baz')

    def test_getitem(self):
        dir = self._makeOne()
        subpath = os.path.join(dir.directory, 'demo')
        self.assertEqual(dir['demo'].directory, subpath)

    def test_contains(self):
        dir = self._makeOne()
        self.assertTrue('demo' in dir)

    def test_openFile(self):
        dir = self._makeOne()
        file = dir.openFile('demo/foo/test.html')
        self.assertEqual('asdf', file.read())

    def test_readFile(self):
        dir = self._makeOne()
        self.assertEqual('asdf', dir.readFile('demo/foo/test.html'))

    def test_readFile_not_found(self):
        dir = self._makeOne()
        self.assertRaises(IOError, dir.readFile, 'baz')

    def test_listDirectory(self):
        dir = self._makeOne()
        self.assertEqual(['demo'], dir.listDirectory())

    def test_listDirectory_filters_by_name(self):
        dir = self._makeOne()
        name = '.dummy'
        file_path = os.path.join(dir.directory, name)
        if name not in os.listdir(dir.directory):
            f = open(file_path, 'w')
            f.write("")
            f.close()
        self.assertTrue(name in os.listdir(dir.directory))
        self.assertEqual(['demo'], dir.listDirectory())
        # Cleanup created file.
        os.remove(file_path)



# -*- coding: utf-8 -*-
from plone.resource.interfaces import IUniqueResourceRequest
from plone.resource.utils import queryResourceDirectory
from zExceptions import NotFound
from zope.interface import alsoProvides
from zope.traversing.namespace import SimpleHandler
import urllib


class ResourceTraverser(SimpleHandler):

    name = None

    def __init__(self, context, request=None):
        self.context = context

    def traverse(self, name, remaining):
        _type = self.name

        # Note: also fixes possible unicode problems
        name = urllib.quote(name)

        res = queryResourceDirectory(_type, name)
        if res is not None:
            return res

        raise NotFound


class UniqueResourceTraverser(SimpleHandler):
    """A traverser to allow unique URLs for caching"""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def traverse(self, name, remaining):
        alsoProvides(self.request, IUniqueResourceRequest)
        return self.context



# -*- coding: utf-8 -*-
import unittest2 as unittest
from plone.testing.zca import UNIT_TESTING

class TestFilesystemResourceDirectory(unittest.TestCase):

    layer = UNIT_TESTING

    def test_render(self):
        import os.path
        from email.utils import formatdate
        from plone.resource.file import FilesystemFile
        from zope.publisher.browser import TestRequest

        name = 'test.html'
        path = os.path.join(os.path.dirname(__file__), 'resources', 'demo', 'foo', name)
        mtime = os.path.getmtime(path)

        request = TestRequest()

        f = FilesystemFile(None, request, path, name)
        iterator = f()

        data = ''.join(iterator)
        self.assertEqual(data, 'asdf')
        self.assertEqual(request.response.getHeader('Content-Type'), 'text/html')
        self.assertEqual(request.response.getHeader('Content-Length'), '4')
        self.assertEqual(request.response.getHeader('Last-Modified'), formatdate(mtime, usegmt=True))

    def test_last_modified(self):
        import os.path
        import datetime

        from dateutil.tz import tzlocal

        from zope.component import provideAdapter
        from zope.publisher.browser import TestRequest

        from plone.resource.file import FilesystemFile
        from plone.resource.file import FileLastModified

        from z3c.caching.interfaces import ILastModified

        provideAdapter(FileLastModified)

        name = 'test.html'
        path = os.path.join(os.path.dirname(__file__), 'resources', 'demo', 'foo', name)
        mtime = os.path.getmtime(path)

        request = TestRequest()

        f = FilesystemFile(None, request, path, name)

        lastModified = ILastModified(f)
        mdate = datetime.datetime.fromtimestamp(mtime, tz=tzlocal())

        self.assertEqual(lastModified(), mdate)

    def test_raw_read_file(self):
        import os.path

        from zope.component import provideAdapter
        from zope.publisher.browser import TestRequest

        from plone.resource.file import FilesystemFile
        from plone.resource.file import rawReadFile

        from zope.filerepresentation.interfaces import IRawReadFile

        provideAdapter(rawReadFile)

        name = 'test.html'
        path = os.path.join(os.path.dirname(__file__), 'resources', 'demo', 'foo', name)

        request = TestRequest()

        f = FilesystemFile(None, request, path, name)

        rf = IRawReadFile(f)

        self.assertTrue(isinstance(rf, file))
        self.assertEqual(rf.read(), 'asdf')

        rf.close()



# -*- coding: utf-8 -*-
import os.path

from zope.interface import Interface
from zope.component.zcml import handler
from zope.configuration.exceptions import ConfigurationError
from zope.configuration.fields import PythonIdentifier
from zope.schema import ASCIILine, TextLine

from plone.resource.interfaces import IResourceDirectory
from plone.resource.directory import FilesystemResourceDirectory


class IResourceDirectoryDirective(Interface):
    """Register resource directories with the global registry.
    """

    directory = TextLine(
        title = u'Directory path',
        description = u'Path relative to the package.',
        required = True
        )

    name = PythonIdentifier(
        title = u'Name',
        description = u'Name of the directory. If not specified, the name of '
                      u'the current package is used.',
        required = False,
        )

    type = ASCIILine(
        title = u'Resource type',
# XXX use a Choice field + vocab
#        vocabulary = 'plone.resource.vocab.ResourceTypes',
        required = False,
        )


def registerResourceDirectory(_context, directory, name=None, type=None):
    """
    Register a new resource directory.

    The actual ZCA registrations are deferred so that conflicts can be resolved
    via zope.configuration's discriminator machinery.
    """

    if _context.package and os.path.isabs(directory):
        raise ConfigurationError('Resource directories in distributions must '
                                 'be specified as relative paths.')
    elif _context.package:
        directory = _context.path(directory)
    elif not _context.package and not os.path.isabs(directory):
        raise ConfigurationError('Global resource directories must be '
                                 'specified as absolute paths.')

    # TODO: make sure this works in Windows
    if '..' in directory.split('/'):
        raise ConfigurationError('Traversing to parent directories '
                                 'via .. is not allowed.')
    if not os.path.exists(directory):
        raise IOError, 'Directory not found: %s' % directory

    if name is None and _context.package:
        name = _context.package.__name__

    if type:
        identifier = '++%s++%s' % (type, name or '')
    else:
        if _context.package:
            raise ConfigurationError('Resource directories in distributions '
                                     'must have a specified resource type.')
        identifier = name or ''

    directory = os.path.sep.join(directory.split('/'))
    directory = FilesystemResourceDirectory(directory, name)

    _context.action(
        discriminator = ('plone:static', identifier),
        callable = handler,
        args = ('registerUtility', directory, IResourceDirectory, identifier),
        )



# -*- coding: utf-8 -*-
from Acquisition import aq_base, aq_parent
from OFS.Image import File
from OFS.interfaces import IObjectManager
from plone.resource.events import PloneResourceCreatedEvent
from plone.resource.events import PloneResourceModifiedEvent
from plone.resource.file import FilesystemFile
from plone.resource.interfaces import IResourceDirectory
from plone.resource.interfaces import IWritableResourceDirectory
from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2
from Products.CMFCore.utils import getToolByName
from StringIO import StringIO
from zExceptions import Forbidden
from zExceptions import NotFound
from zope.event import notify
from zope.interface import implementer
from zope.site.hooks import getSite
import os.path
import re
import zipfile

# filter dot files, Mac resource forks
FILTERS = (r'\..*', '__MACOSX')
FILTERS = [re.compile(pattern) for pattern in FILTERS]


@implementer(IWritableResourceDirectory)
class PersistentResourceDirectory(object):
    """A resource directory stored in the ZODB.

    It is assumed that directories provide IObjectManager
    and that files are instances of OFS.Image.File.
    """

    def __init__(self, context=None):
        if context is None:
            # This is also used as a local IResourceDirectory utility,
            # named u'persistent', which wraps the root folder.
            # This gets pickled, so we can't keep the acquisition chain.
            context = aq_base(getToolByName(getSite(), 'portal_resources'))
        self.context = self.__parent__ = context
        self.__name__ = context.getId()

    def __repr__(self):
        return '<%s object at %s>' % (self.__class__.__name__,
                                      '/'.join(self.context.getPhysicalPath()))

    def publishTraverse(self, request, name):
        if isinstance(name, unicode):
            name = name.encode('utf-8')

        context = self.context
        if aq_parent(context) is None:
            # Re-supply the acquisition chain if this is the root resource
            # directory, so that security checks work.
            site = getSite()
            if site is not None:
                context = context.__of__(site)

        if self.isDirectory(name):
            return self.__class__(
                context.unrestrictedTraverse(name).__of__(context)
            )
        elif self.isFile(name):
            return context.unrestrictedTraverse(name).__of__(context)

        raise NotFound

    def __getitem__(self, name):
        return self.publishTraverse(None, name)

    def __setitem__(self, name, item):
        if isinstance(name, unicode):
            name = name.encode('utf-8')

        if IResourceDirectory.providedBy(item):
            item = item.context
        self.context[name] = item
        item.id = item.__name__ = name

    def __delitem__(self, name):
        del self.context[name]

    def __contains__(self, name):
        return name in self.context

    def openFile(self, path):
        return StringIO(self.readFile(path))

    def readFile(self, path):
        try:
            f = self.context.unrestrictedTraverse(path)
        except Exception as e:
            raise IOError(str(e))

        return str(f.data)

    def listDirectory(self):
        return [n for n in self.context.objectIds()
                if not any(filter.match(n) for filter in FILTERS)]

    def isDirectory(self, path):
        try:
            obj = self.context.unrestrictedTraverse(path)
        except:
            obj = None

        return IObjectManager.providedBy(obj)

    def isFile(self, path):
        try:
            obj = self.context.unrestrictedTraverse(path)
        except:
            obj = None

        return isinstance(obj, File)

    def rename(self, oldName, newName):
        obj = self.context[oldName]
        obj.id = obj.__name__ = newName
        self.context._delOb(oldName)
        self.context._setOb(newName, obj)

    def exportZip(self, out):
        prefix = self.__name__

        zf = zipfile.ZipFile(out, 'w')

        def write(dir, prefix, zf):
            for name in dir.listDirectory():
                relativeName = "%s/%s" % (prefix, name,)
                if dir.isDirectory(name):
                    write(dir[name], relativeName, zf)
                elif dir.isFile(name):
                    zf.writestr(relativeName, dir.readFile(name))

        write(self, prefix, zf)
        zf.close()

    def makeDirectory(self, path):
        parent = self.context
        names = path.strip('/').split('/')
        for name in names:
            if name not in parent:
                if isinstance(name, unicode):
                    name = name.encode('utf-8')
                f = BTreeFolder2(name)
                parent._setOb(name, f)
            parent = parent[name]

    def writeFile(self, path, data):
        basepath = '/'.join(path.split('/')[:-1])
        if basepath:
            self.makeDirectory(basepath)
        filename = path.split('/')[-1]
        if isinstance(filename, unicode):
            filename = filename.encode('utf-8')
        f = File(filename, filename, data)
        ct = f.getContentType()
        if ct.startswith('text/') or ct == 'application/javascript':
            # otherwise HTTPResponse.setBody assumes latin1 and mangles things
            f.content_type = ct + '; charset=utf-8'
        container = self.context.unrestrictedTraverse(basepath)
        if filename in container:
            container._delOb(filename)
            event = PloneResourceModifiedEvent
        else:
            event = PloneResourceCreatedEvent
        container._setOb(filename, f)
        obj = container._getOb(filename)
        notify(event(obj))

    def importZip(self, f):
        if not isinstance(f, zipfile.ZipFile):
            f = zipfile.ZipFile(f)
        for name in f.namelist():
            member = f.getinfo(name)
            path = member.filename.lstrip('/')

            # test each part of the path against the filters
            if any(any(filter.match(n) for filter in FILTERS)
                   for n in path.split('/')
                   ):
                continue

            if path.endswith('/'):
                self.makeDirectory(path)
            else:
                data = f.open(member).read()
                self.writeFile(path, data)


@implementer(IResourceDirectory)
class FilesystemResourceDirectory(object):
    """A resource directory based on files in the filesystem.
    """

    __allow_access_to_unprotected_subobjects__ = True

    def __init__(self, directory, name=None, parent=None):
        self.directory = directory
        self.__name__ = name
        if name is None:
            self.__name__ = os.path.basename(directory)
        self._parent = parent

    @property
    def __parent__(self):
        if self._parent is None:
            return getSite()
        return self._parent

    @__parent__.setter
    def __parent__(self, value):
        self._parent = value

    def __repr__(self):
        return '<%s object at %s>' % (self.__class__.__name__, self.__name__)

    def _resolveSubpath(self, path):
        parts = path.split('/')
        filepath = os.path.abspath(os.path.join(self.directory, *parts))
        if not filepath.startswith(self.directory):
            raise Forbidden('Invalid path resource')
        return filepath

    def publishTraverse(self, request, name):
        filepath = self._resolveSubpath(name)
        if self.isDirectory(name):
            return self.__class__(filepath, parent=self)
        elif self.isFile(name):
            return FilesystemFile(self, request, filepath, name)

        raise NotFound

    def __contains__(self, name):
        if self.publishTraverse(None, name):
            return True
        return False

    def __getitem__(self, name):
        return self.publishTraverse(None, name)

    def openFile(self, path):
        filepath = self._resolveSubpath(path)
        return open(filepath, 'rb')

    def readFile(self, path):
        return self.openFile(path).read()

    def listDirectory(self):
        names = os.listdir(self.directory)
        return [n for n in names
                if not any(filter.match(n) for filter in FILTERS)]

    def isDirectory(self, path):
        return os.path.isdir(self._resolveSubpath(path))

    def isFile(self, path):
        return os.path.isfile(self._resolveSubpath(path))

    def exportZip(self, out):
        zf = zipfile.ZipFile(out, 'w')

        toStrip = len(self.directory.replace(os.path.sep, '/')) + 1

        for (dirpath, dirnames, filenames) in os.walk(self.directory):
            subpath = dirpath.replace(os.path.sep, '/')[toStrip:].strip('/')

            for filename in filenames:
                path = '/'.join([subpath, filename]).strip('/')

                if any(any(filter.match(n) for filter in FILTERS)
                       for n in path.split('/')
                       ):
                    continue

                zf.writestr(
                    '/'.join([self.__name__, path, ]),
                    self.readFile(path)
                )
        zf.close()



# -*- coding: utf-8 -*-
from plone.caching.interfaces import IRulesetLookup
from plone.resource.interfaces import IUniqueResourceRequest
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(IRulesetLookup)
@adapter(Interface, IUniqueResourceRequest)
class UniqueResourceLookup(object):
    """Unique resource ruleset lookup.

    Returns 'plone.stableResource' for requests marked with
    IUniqueResourceRequest.
    """

    def __init__(self, published, request):
        pass

    def __call__(self):
        return 'plone.stableResource'



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = '1.2.2'

setup(
    name='plone.resource',
    version=version,
    description="Static files for Plone",
    long_description=(
        open("README.rst").read() +
        "\n" +
        open("CHANGES.rst").read()
    ),
    classifiers=[
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone :: 5.1",
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
    ],
    keywords='plone resource',
    author='David Glick, Plone Foundation',
    author_email='davidglick@groundwire.org',
    url='https://pypi.python.org/pypi/plone.resource',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['plone'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'plone.caching',
        'python-dateutil',
        'setuptools',
        'z3c.caching',
        'zope.component',
        'zope.configuration',
        'zope.filerepresentation',
        'zope.interface',
        'zope.publisher',
        'zope.schema',
        'zope.traversing',
        'Zope2',
    ],
    extras_require={
        'test': [
	    'plone.app.testing',
            'unittest2',
        ]
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
import unittest2 as unittest
from plone.testing import zca
from plone.resource.testing import DEMO_TRAVERSER_INTEGRATION_TESTING

import zipfile
import os.path

from StringIO import StringIO

from zope.component import provideUtility

from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2
from OFS.Image import File

from plone.resource.interfaces import IResourceDirectory
from plone.resource.directory import PersistentResourceDirectory
from plone.resource.directory import FilesystemResourceDirectory

base_path = os.path.dirname(__file__)
test_dir_path = os.path.join(base_path, 'resources')


class ZipDownloadTestCase(unittest.TestCase):
    layer = DEMO_TRAVERSER_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer.get('app')
        zca.pushGlobalRegistry()

    def tearDown(self):
        zca.popGlobalRegistry()

    def test_traverse_packaged_type_specific_directory(self):
        dir = FilesystemResourceDirectory(os.path.join(test_dir_path, 'demo', 'foo'))
        provideUtility(dir, provides=IResourceDirectory, name=u'++demo++foo')

        out = StringIO()
        request = self.layer['request']
        response = request.response
        response.stdout = out

        zipview = self.app.unrestrictedTraverse('++demo++foo/@@download-zip')
        zipview()

        zf = zipfile.ZipFile(out)

        self.assertTrue('foo/test.html' in zf.namelist())
        self.assertEqual('asdf', zf.open('foo/test.html').read())

    def test_traverse_global_directory(self):
        dir = FilesystemResourceDirectory(test_dir_path)
        provideUtility(dir, provides=IResourceDirectory, name=u'')

        out = StringIO()
        request = self.layer['request']
        response = request.response
        response.stdout = out

        zipview = self.app.unrestrictedTraverse('++demo++foo/@@download-zip')
        zipview()

        zf = zipfile.ZipFile(out)

        self.assertTrue('foo/test.html' in zf.namelist())
        self.assertEqual('asdf', zf.open('foo/test.html').read())

    def test_traverse_persistent_directory(self):
        root = BTreeFolder2('portal_resources')
        self.app._setOb('portal_resources', root)
        root._setOb('demo', BTreeFolder2('demo'))
        root['demo']._setOb('foo', BTreeFolder2('foo'))
        root['demo']['foo']._setOb('test.html', File('test.html', 'test.html', 'asdf'))

        dir = PersistentResourceDirectory(root)
        provideUtility(dir, provides=IResourceDirectory, name=u'persistent')

        out = StringIO()
        request = self.layer['request']
        response = request.response
        response.stdout = out

        zipview = self.app.unrestrictedTraverse('++demo++foo/@@download-zip')
        zipview()

        zf = zipfile.ZipFile(out)

        self.assertTrue('foo/test.html' in zf.namelist())
        self.assertEqual('asdf', zf.open('foo/test.html').read())



# -*- coding: utf-8 -*-
from plone.app.testing import applyProfile
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.resource.traversal import ResourceTraverser
from plone.testing import Layer
from plone.testing import publisher
from plone.testing import z2
from plone.testing import zca
from zope.configuration import xmlconfig


class DemoTraverser(Layer):
    defaultBases = (z2.STARTUP, publisher.PUBLISHER_DIRECTIVES)

    def setUp(self):
        # Stack a new configuration context
        self['configurationContext'] = context = zca.stackConfigurationContext(
            self.get('configurationContext')
        )

        import plone.resource
        xmlconfig.file('testing.zcml', plone.resource, context=context)

    def tearDown(self):
        # Zap the stacked configuration context
        del self['configurationContext']

DEMO_TRAVERSER_FIXTURE = DemoTraverser()
DEMO_TRAVERSER_INTEGRATION_TESTING = z2.IntegrationTesting(
    bases=(DEMO_TRAVERSER_FIXTURE,),
    name="plone.resource:DemoTraverser"
)


class PloneResource(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import plone.resource
        xmlconfig.file(
            'configure.zcml',
            plone.resource,
            context=configurationContext
        )

    def setUpPloneSite(self, portal):
        # install into the Plone site
        applyProfile(portal, 'plone.resource:default')

        from transaction import commit
        commit()

PLONE_RESOURCE_FIXTURE = PloneResource()
PLONE_RESOURCE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_RESOURCE_FIXTURE,),
    name="plone.resource:Integration"
)


class DemoTraverser(ResourceTraverser):
    name = 'demo'



# -*- coding: utf-8 -*-
from datetime import time
from dateutil.tz import tzlocal
from email.utils import formatdate
from z3c.caching.interfaces import ILastModified
from zope.component import adapter
from zope.filerepresentation.interfaces import IRawReadFile
from zope.interface import implementer
from ZPublisher.Iterators import filestream_iterator
from zope.component import queryUtility

import datetime
import mimetypes
import os
import os.path


class ResourceIterator(filestream_iterator):
    """Resource iterator that allows (inefficient) coercion to str/unicode.

    This is needed for ResourceRegistries support, for example.
    """

    def __str__(self):
        return self.read()

    def __unicode__(self):
        return self.read().decode('utf-8')


class FilesystemFile(object):
    """Representation of a file. When called, it will set response headers
    and return the file's contents
    """

    def __init__(self, parent, request, path, name):
        self.path = path
        self.request = request
        self.__name__ = name
        self.__parent__ = parent
        self.lastModifiedTimestamp = float(
            os.path.getmtime(path)
        ) or time.time()

    def getContentType(self, default='application/octet-stream'):
        extension = os.path.splitext(self.__name__)[1].lower()
        mtr = queryUtility('mimetypes_registry')
        mt = None
        if mtr:
            mt = mtr.lookupExtension(extension)
        if mt is None:
            mt = mimetypes.types_map.get(extension, default)
        return mt

    def __call__(self, REQUEST=None, RESPONSE=None):
        contentType = self.getContentType()
        lastModifiedHeader = formatdate(
            self.lastModifiedTimestamp,
            usegmt=True
        )

        request = REQUEST
        if request is None:
            request = self.request

        response = RESPONSE
        if response is None:
            response = self.request.response

        response.setHeader('Content-Type', contentType)
        response.setHeader('Content-Length', os.path.getsize(self.path))
        response.setHeader('Last-Modified', lastModifiedHeader)

        return ResourceIterator(self.path, 'rb')


@implementer(ILastModified)
@adapter(FilesystemFile)
class FileLastModified(object):
    """Determine when a file was last modified, for caching purposes
    """

    def __init__(self, context):
        self.context = context

    def __call__(self):
        return datetime.datetime.fromtimestamp(
            self.context.lastModifiedTimestamp,
            tz=tzlocal()
        )


@implementer(IRawReadFile)
@adapter(FilesystemFile)
def rawReadFile(context):
    return open(context.path, 'rb')
